@@ -5,6 +5,12 @@ define('XMLRPC_REQUEST', true);
 // Some browser-embedded clients send cookies. We don't want them.
 $_COOKIE = array();
 
+// A bug in PHP < 5.2.2 makes $HTTP_RAW_POST_DATA not set by default, 
+// but we can do it ourself.
+if ( !isset( $HTTP_RAW_POST_DATA ) ) {
+	$HTTP_RAW_POST_DATA = file_get_contents( 'php://input' );
+}
+
 # fix for mozBlog and other cases where '<?xml' isn't on the very first line
 if ( isset($HTTP_RAW_POST_DATA) )
 	$HTTP_RAW_POST_DATA = trim($HTTP_RAW_POST_DATA);
@@ -22,10 +28,10 @@ header('Content-type: text/xml; charset=' . get_option('blog_charset'), true);
     <engineLink>http://wordpress.org/</engineLink>
     <homePageLink><?php bloginfo_rss('url') ?></homePageLink>
     <apis>
-      <api name="WordPress" blogID="1" preferred="false" apiLink="<?php bloginfo_rss('url') ?>/xmlrpc.php" />
-      <api name="Movable Type" blogID="1" preferred="true" apiLink="<?php bloginfo_rss('url') ?>/xmlrpc.php" />
-      <api name="MetaWeblog" blogID="1" preferred="false" apiLink="<?php bloginfo_rss('url') ?>/xmlrpc.php" />
-      <api name="Blogger" blogID="1" preferred="false" apiLink="<?php bloginfo_rss('url') ?>/xmlrpc.php" />
+      <api name="WordPress" blogID="1" preferred="false" apiLink="<?php bloginfo_rss('wpurl') ?>/xmlrpc.php" />
+      <api name="Movable Type" blogID="1" preferred="true" apiLink="<?php bloginfo_rss('wpurl') ?>/xmlrpc.php" />
+      <api name="MetaWeblog" blogID="1" preferred="false" apiLink="<?php bloginfo_rss('wpurl') ?>/xmlrpc.php" />
+      <api name="Blogger" blogID="1" preferred="false" apiLink="<?php bloginfo_rss('wpurl') ?>/xmlrpc.php" />
     </apis>
   </service>
 </rsd>
@@ -140,7 +146,7 @@ class wp_xmlrpc_server extends IXR_Server {
 
 	function login_pass_ok($user_login, $user_pass) {
 		if (!user_pass_ok($user_login, $user_pass)) {
-			$this->error = new IXR_Error(403, 'Bad login/pass combination.');
+			$this->error = new IXR_Error(403, __('Bad login/pass combination.'));
 			return false;
 		}
 		return true;
@@ -202,7 +208,7 @@ class wp_xmlrpc_server extends IXR_Server {
 			$allow_pings = ("open" == $page->ping_status) ? 1 : 0;
 
 			// Format page date.
-			$page_date = mysql2date("Ymd\TH:i:s", $page->post_date);
+			$page_date = mysql2date("Ymd\TH:i:s\Z", $page->post_date_gmt);
 
 			// Pull the categories info together.
 			$categories = array();
@@ -241,7 +247,7 @@ class wp_xmlrpc_server extends IXR_Server {
 		}
 		// If the page doesn't exist indicate that.
 		else {
-			return(new IXR_Error(404, "Sorry, no such page."));
+			return(new IXR_Error(404, __("Sorry, no such page.")));
 		}
 	}
 
@@ -302,7 +308,7 @@ class wp_xmlrpc_server extends IXR_Server {
 		// to add new pages.
 		$user = set_current_user(0, $username);
 		if(!current_user_can("publish_pages")) {
-			return(new IXR_Error(401, "Sorry, you can not add new pages."));
+			return(new IXR_Error(401, __("Sorry, you can not add new pages.")));
 		}
 
 		// Mark this as content for a page.
@@ -335,19 +341,19 @@ class wp_xmlrpc_server extends IXR_Server {
 			!$actual_page
 			|| ($actual_page["post_type"] != "page")
 		) {
-			return(new IXR_Error(404, "Sorry, no such page."));
+			return(new IXR_Error(404, __("Sorry, no such page.")));
 		}
 
 		// Set the user context and make sure they can delete pages.
 		set_current_user(0, $username);
 		if(!current_user_can("delete_page", $page_id)) {
-			return(new IXR_Error(401, "Sorry, you do not have the right to delete this page."));
+			return(new IXR_Error(401, __("Sorry, you do not have the right to delete this page.")));
 		}
 
 		// Attempt to delete the page.
 		$result = wp_delete_post($page_id);
 		if(!$result) {
-			return(new IXR_Error(500, "Failed to delete the page."));
+			return(new IXR_Error(500, __("Failed to delete the page.")));
 		}
 
 		return(true);
@@ -360,7 +366,7 @@ class wp_xmlrpc_server extends IXR_Server {
 	function wp_editPage($args) {
 		// Items not escaped here will be escaped in editPost.
 		$blog_id	= (int) $args[0];
-		$page_id	= $this->escape((int) $args[1]);
+		$page_id	= (int) $this->escape($args[1]);
 		$username	= $this->escape($args[2]);
 		$password	= $this->escape($args[3]);
 		$content	= $args[4];
@@ -376,13 +382,13 @@ class wp_xmlrpc_server extends IXR_Server {
 			!$actual_page
 			|| ($actual_page["post_type"] != "page")
 		) {
-			return(new IXR_Error(404, "Sorry, no such page."));
+			return(new IXR_Error(404, __("Sorry, no such page.")));
 		}
 
 		// Set the user context and make sure they are allowed to edit pages.
 		set_current_user(0, $username);
 		if(!current_user_can("edit_page", $page_id)) {
-			return(new IXR_Error(401, "Sorry, you do not have the right to edit this page."));
+			return(new IXR_Error(401, __("Sorry, you do not have the right to edit this page.")));
 		}
 
 		// Mark this as content for a page.
@@ -423,7 +429,7 @@ class wp_xmlrpc_server extends IXR_Server {
 			SELECT ID page_id,
 				post_title page_title,
 				post_parent page_parent_id,
-				post_date
+				post_date_gmt
 			FROM {$wpdb->posts}
 			WHERE post_type = 'page'
 			ORDER BY ID
@@ -432,10 +438,10 @@ class wp_xmlrpc_server extends IXR_Server {
 		// The date needs to be formated properly.
 		$num_pages = count($page_list);
 		for($i = 0; $i < $num_pages; $i++) {
-			$post_date = mysql2date("Ymd\TH:i:s", $page_list[$i]->post_date);
+			$post_date = mysql2date("Ymd\TH:i:s\Z", $page_list[$i]->post_date_gmt);
 			$page_list[$i]->dateCreated = new IXR_Date($post_date);
 
-			unset($page_list[$i]->post_date);
+			unset($page_list[$i]->post_date_gmt);
 		}
 
 		return($page_list);
@@ -481,7 +487,7 @@ class wp_xmlrpc_server extends IXR_Server {
 		// allowed to add a category.
 		set_current_user(0, $username);
 		if(!current_user_can("manage_categories", $page_id)) {
-			return(new IXR_Error(401, "Sorry, you do not have the right to add a category."));
+			return(new IXR_Error(401, __("Sorry, you do not have the right to add a category.")));
 		}
 
 		// We need this to make use of the wp_insert_category()
@@ -513,7 +519,7 @@ class wp_xmlrpc_server extends IXR_Server {
 
 		$cat_id = wp_insert_category($new_category);
 		if(!$cat_id) {
-			return(new IXR_Error(500, "Sorry, the new category failed."));
+			return(new IXR_Error(500, __("Sorry, the new category failed.")));
 		}
 
 		return($cat_id);
@@ -532,7 +538,7 @@ class wp_xmlrpc_server extends IXR_Server {
 		$username				= $args[1];
 		$password				= $args[2];
 		$category				= $args[3];
-		$max_results			= $args[4];
+		$max_results			= (int) $args[4];
 
 		if(!$this->login_pass_ok($username, $password)) {
 			return($this->error);
@@ -569,8 +575,9 @@ class wp_xmlrpc_server extends IXR_Server {
 		$user_login = $args[1];
 		$user_pass  = $args[2];
 
-		if (!$this->login_pass_ok($user_login, $user_pass))
+		if (!$this->login_pass_ok($user_login, $user_pass)) {
 			return $this->error;
+		}
 
 		$user = set_current_user(0, $user_login);
 
@@ -591,9 +598,11 @@ class wp_xmlrpc_server extends IXR_Server {
 				'blogid'   => $blog_id,
 				'blogName' => get_option('blogname')
 			);
+
+			restore_current_blog();
 		}
 
-	  return array($struct);
+	  return $struct;
 	}
 
 
@@ -676,7 +685,7 @@ class wp_xmlrpc_server extends IXR_Server {
 		$posts_list = wp_get_recent_posts($num_posts);
 
 		if (!$posts_list) {
-			$this->error = new IXR_Error(500, 'Either there are no posts, or something went wrong.');
+			$this->error = new IXR_Error(500, __('Either there are no posts, or something went wrong.'));
 			return $this->error;
 		}
 
@@ -723,7 +732,7 @@ class wp_xmlrpc_server extends IXR_Server {
 
 	  set_current_user(0, $user_login);
 	  if ( !current_user_can('edit_themes') ) {
-	    return new IXR_Error(401, 'Sorry, this user can not edit the template.');
+	    return new IXR_Error(401, __('Sorry, this user can not edit the template.'));
 	  }
 
 	  /* warning: here we make the assumption that the weblog's URL is on the same server */
@@ -758,7 +767,7 @@ class wp_xmlrpc_server extends IXR_Server {
 
 	  set_current_user(0, $user_login);
 	  if ( !current_user_can('edit_themes') ) {
-	    return new IXR_Error(401, 'Sorry, this user can not edit the template.');
+	    return new IXR_Error(401, __('Sorry, this user can not edit the template.'));
 	  }
 
 	  /* warning: here we make the assumption that the weblog's URL is on the same server */
@@ -769,7 +778,7 @@ class wp_xmlrpc_server extends IXR_Server {
 	    fwrite($f, $content);
 	    fclose($f);
 	  } else {
-	    return new IXR_Error(500, 'Either the file is not writable, or something wrong happened. The file has not been updated.');
+	    return new IXR_Error(500, __('Either the file is not writable, or something wrong happened. The file has not been updated.'));
 	  }
 
 	  return true;
@@ -796,7 +805,7 @@ class wp_xmlrpc_server extends IXR_Server {
 	  $cap = ($publish) ? 'publish_posts' : 'edit_posts';
 	  $user = set_current_user(0, $user_login);
 	  if ( !current_user_can($cap) )
-	    return new IXR_Error(401, 'Sorry, you can not post on this weblog or category.');
+	    return new IXR_Error(401, __('Sorry, you can not post on this weblog or category.'));
 
 	  $post_status = ($publish) ? 'publish' : 'draft';
 
@@ -814,7 +823,7 @@ class wp_xmlrpc_server extends IXR_Server {
 	  $post_ID = wp_insert_post($post_data);
 
 	  if (!$post_ID) {
-	    return new IXR_Error(500, 'Sorry, your entry could not be posted. Something wrong happened.');
+	    return new IXR_Error(500, __('Sorry, your entry could not be posted. Something wrong happened.'));
 	  }
 	  $this->attach_uploads( $post_ID, $post_content );
 
@@ -844,19 +853,19 @@ class wp_xmlrpc_server extends IXR_Server {
 	  $actual_post = wp_get_single_post($post_ID,ARRAY_A);
 
 	  if (!$actual_post) {
-	  	return new IXR_Error(404, 'Sorry, no such post.');
+	  	return new IXR_Error(404, __('Sorry, no such post.'));
 	  }
 
 		$this->escape($actual_post);
 
 	  set_current_user(0, $user_login);
 	  if ( !current_user_can('edit_post', $post_ID) )
-	    return new IXR_Error(401, 'Sorry, you do not have the right to edit this post.');
+	    return new IXR_Error(401, __('Sorry, you do not have the right to edit this post.'));
 
-	  extract($actual_post);
+	  extract($actual_post, EXTR_SKIP);
 
 	  if ( ('publish' == $post_status) && !current_user_can('publish_posts') )
-	  	return new IXR_Error(401, 'Sorry, you do not have the right to publish this post.');
+	  	return new IXR_Error(401, __('Sorry, you do not have the right to publish this post.'));
 
 	  $post_title = xmlrpc_getposttitle($content);
 	  $post_category = xmlrpc_getpostcategory($content);
@@ -867,7 +876,7 @@ class wp_xmlrpc_server extends IXR_Server {
 	  $result = wp_update_post($postdata);
 
 	  if (!$result) {
-	  	return new IXR_Error(500, 'For some strange yet very annoying reason, this post could not be edited.');
+	  	return new IXR_Error(500, __('For some strange yet very annoying reason, this post could not be edited.'));
 	  }
 	  $this->attach_uploads( $ID, $post_content );
 
@@ -894,17 +903,17 @@ class wp_xmlrpc_server extends IXR_Server {
 	  $actual_post = wp_get_single_post($post_ID,ARRAY_A);
 
 	  if (!$actual_post) {
-	  	return new IXR_Error(404, 'Sorry, no such post.');
+	  	return new IXR_Error(404, __('Sorry, no such post.'));
 	  }
 
 	  set_current_user(0, $user_login);
 	  if ( !current_user_can('edit_post', $post_ID) )
-	    return new IXR_Error(401, 'Sorry, you do not have the right to delete this post.');
+	    return new IXR_Error(401, __('Sorry, you do not have the right to delete this post.'));
 
 	  $result = wp_delete_post($post_ID);
 
 	  if (!$result) {
-	  	return new IXR_Error(500, 'For some strange yet very annoying reason, this post could not be deleted.');
+	  	return new IXR_Error(500, __('For some strange yet very annoying reason, this post could not be deleted.'));
 	  }
 
 	  return true;
@@ -933,9 +942,10 @@ class wp_xmlrpc_server extends IXR_Server {
 	    return $this->error;
 	  }
 
+      $cap = ($publish) ? 'publish_posts' : 'edit_posts';
 	  $user = set_current_user(0, $user_login);
-	  if ( !current_user_can('publish_posts') )
-	    return new IXR_Error(401, 'Sorry, you can not post on this weblog or category.');
+	  if ( !current_user_can($cap) )
+	    return new IXR_Error(401, __('Sorry, you can not post on this weblog or category.'));
 
 		// The post_type defaults to post, but could also be page.
 		$post_type = "post";
@@ -949,12 +959,12 @@ class wp_xmlrpc_server extends IXR_Server {
 		// Let WordPress generate the post_name (slug) unless
 		// one has been provided.
 		$post_name = "";
-		if(!empty($content_struct["wp_slug"])) {
+		if(isset($content_struct["wp_slug"])) {
 			$post_name = $content_struct["wp_slug"];
 		}
 
 		// Only use a password if one was given.
-		if(!empty($content_struct["wp_password"])) {
+		if(isset($content_struct["wp_password"])) {
 			$post_password = $content_struct["wp_password"];
 		}
 
@@ -964,14 +974,17 @@ class wp_xmlrpc_server extends IXR_Server {
 		}
 
 		// Only set the menu_order if it was provided.
-		if(!empty($content_struct["wp_page_order"])) {
+		if(isset($content_struct["wp_page_order"])) {
 			$menu_order = $content_struct["wp_page_order"];
 		}
 
 	  $post_author = $user->ID;
 
 		// If an author id was provided then use it instead.
-		if(!empty($content_struct["wp_author_id"])) {
+		if(
+			isset($content_struct["wp_author_id"])
+			&& ($user->ID != $content_struct["wp_author_id"])
+		) {
 			switch($post_type) {
 				case "post":
 					if(!current_user_can("edit_others_posts")) {
@@ -986,7 +999,7 @@ class wp_xmlrpc_server extends IXR_Server {
 					}
 					break;
 				default:
-					return(new IXR_Error(401, "Invalid post type."));
+					return(new IXR_Error(401, __("Invalid post type.")));
 					break;
 			}
 			$post_author = $content_struct["wp_author_id"];
@@ -999,13 +1012,69 @@ class wp_xmlrpc_server extends IXR_Server {
 	  $post_excerpt = $content_struct['mt_excerpt'];
 	  $post_more = $content_struct['mt_text_more'];
 
-	  $comment_status = (!isset($content_struct['mt_allow_comments'])) ?
-	    get_option('default_comment_status')
-	    : $content_struct['mt_allow_comments'];
+		if(isset($content_struct["mt_allow_comments"])) {
+			if(!is_numeric($content_struct["mt_allow_comments"])) {
+				switch($content_struct["mt_allow_comments"]) {
+					case "closed":
+						$comment_status = "closed";
+						break;
+					case "open":
+						$comment_status = "open";
+						break;
+					default:
+						$comment_status = get_option("default_comment_status");
+						break;
+				}
+			}
+			else {
+				switch((int) $content_struct["mt_allow_comments"]) {
+					case 0:
+						$comment_status = "closed";
+						break;
+					case 1:
+						$comment_status = "open";
+						break;
+					default:
+						$comment_status = get_option("default_comment_status");
+						break;
+				}
+			}
+		}
+		else {
+			$comment_status = get_option("default_comment_status");
+		}
 
-	  $ping_status = (!isset($content_struct['mt_allow_pings'])) ?
-	    get_option('default_ping_status')
-	    : $content_struct['mt_allow_pings'];
+		if(isset($content_struct["mt_allow_pings"])) {
+			if(!is_numeric($content_struct["mt_allow_pings"])) {
+				switch($content["mt_allow_pings"]) {
+					case "closed":
+						$ping_status = "closed";
+						break;
+					case "open":
+						$ping_status = "open";
+						break;
+					default:
+						$ping_status = get_option("default_ping_status");
+						break;
+				}
+			}
+			else {
+				switch((int) $content_struct["mt_allow_pings"]) {
+					case 0:
+						$ping_status = "closed";
+						break;
+					case 1:
+						$ping_status = "open";
+						break;
+					default:
+						$ping_status = get_option("default_ping_status");
+						break;
+				}
+			}
+		}
+		else {
+			$ping_status = get_option("default_ping_status");
+		}
 
 	  if ($post_more) {
 	    $post_content = $post_content . "\n<!--more-->\n" . $post_more;
@@ -1020,7 +1089,7 @@ class wp_xmlrpc_server extends IXR_Server {
 	  if (!empty($dateCreatedd)) {
 	    $dateCreated = $dateCreatedd->getIso();
 	    $post_date     = get_date_from_gmt(iso8601_to_datetime($dateCreated));
-	    $post_date_gmt = iso8601_to_datetime($dateCreated, GMT);
+	    $post_date_gmt = iso8601_to_datetime($dateCreated. "Z", GMT);
 	  } else {
 	    $post_date     = current_time('mysql');
 	    $post_date_gmt = current_time('mysql', 1);
@@ -1042,7 +1111,7 @@ class wp_xmlrpc_server extends IXR_Server {
 	  $post_ID = wp_insert_post($postdata);
 
 	  if (!$post_ID) {
-	    return new IXR_Error(500, 'Sorry, your entry could not be posted. Something wrong happened.');
+	    return new IXR_Error(500, __('Sorry, your entry could not be posted. Something wrong happened.'));
 	  }
 
 	  $this->attach_uploads( $post_ID, $post_content );
@@ -1083,7 +1152,7 @@ class wp_xmlrpc_server extends IXR_Server {
 	    return $this->error;
 	  }
 
-	  set_current_user(0, $user_login);
+		$user = set_current_user(0, $user_login);
 
 		// The post_type defaults to post, but could also be page.
 		$post_type = "post";
@@ -1096,7 +1165,7 @@ class wp_xmlrpc_server extends IXR_Server {
 
 	  // Edit page caps are checked in editPage.  Just check post here.
 	  if ( ( 'post' == $post_type ) && !current_user_can('edit_post', $post_ID) )
-	    return new IXR_Error(401, 'Sorry, you can not edit this post.');
+	    return new IXR_Error(401, __('Sorry, you can not edit this post.'));
 
 	  $postdata = wp_get_single_post($post_ID, ARRAY_A);
 
@@ -1104,20 +1173,20 @@ class wp_xmlrpc_server extends IXR_Server {
 		// now and return an error.  Other wise a new post will be
 		// created (which was the old behavior).
 		if(empty($postdata["ID"])) {
-			return(new IXR_Error(404, "Invalid post id."));
+			return(new IXR_Error(404, __("Invalid post id.")));
 		}
 
-	  extract($postdata);
 		$this->escape($postdata);
+		extract($postdata, EXTR_SKIP);
 
 		// Let WordPress manage slug if none was provided.
 		$post_name = "";
-		if(!empty($content_struct["wp_slug"])) {
+		if(isset($content_struct["wp_slug"])) {
 			$post_name = $content_struct["wp_slug"];
 		}
 
 		// Only use a password if one was given.
-		if(!empty($content_struct["wp_password"])) {
+		if(isset($content_struct["wp_password"])) {
 			$post_password = $content_struct["wp_password"];
 		}
 
@@ -1127,12 +1196,17 @@ class wp_xmlrpc_server extends IXR_Server {
 		}
 
 		// Only set the menu_order if it was given.
-		if(!empty($content_struct["wp_page_order"])) {
+		if(isset($content_struct["wp_page_order"])) {
 			$menu_order = $content_struct["wp_page_order"];
 		}
 
+		$post_author = $postdata["post_author"];
+
 		// Only set the post_author if one is set.
-		if(!empty($content_struct["wp_author_id"])) {
+		if(
+			isset($content_struct["wp_author_id"])
+			&& ($user->ID != $content_struct["wp_author_id"])
+		) {
 			switch($post_type) {
 				case "post":
 					if(!current_user_can("edit_others_posts")) {
@@ -1147,21 +1221,67 @@ class wp_xmlrpc_server extends IXR_Server {
 					}
 					break;
 				default:
-					return(new IXR_Error(401, "Invalid post type."));
+					return(new IXR_Error(401, __("Invalid post type.")));
 					break;
 			}
 			$post_author = $content_struct["wp_author_id"];
 		}
 
-		// Only set ping_status if it was provided.
+		if(isset($content_struct["mt_allow_comments"])) {
+			if(!is_numeric($content_struct["mt_allow_comments"])) {
+				switch($content_struct["mt_allow_comments"]) {
+					case "closed":
+						$comment_status = "closed";
+						break;
+					case "open":
+						$comment_status = "open";
+						break;
+					default:
+						$comment_status = get_option("default_comment_status");
+						break;
+				}
+			}
+			else {
+				switch((int) $content_struct["mt_allow_comments"]) {
+					case 0:
+						$comment_status = "closed";
+						break;
+					case 1:
+						$comment_status = "open";
+						break;
+					default:
+						$comment_status = get_option("default_comment_status");
+						break;
+				}
+			}
+		}
+
 		if(isset($content_struct["mt_allow_pings"])) {
-			switch($content_struct["mt_allow_pings"]) {
-				case "0":
-					$ping_status = "closed";
-					break;
-				case "1":
-					$ping_status = "open";
-					break;
+			if(!is_numeric($content_struct["mt_allow_pings"])) {
+				switch($content["mt_allow_pings"]) {
+					case "closed":
+						$ping_status = "closed";
+						break;
+					case "open":
+						$ping_status = "open";
+						break;
+					default:
+						$ping_status = get_option("default_ping_status");
+						break;
+				}
+			}
+			else {
+				switch((int) $content_struct["mt_allow_pings"]) {
+					case 0:
+						$ping_status = "closed";
+						break;
+					case 1:
+						$ping_status = "open";
+						break;
+					default:
+						$ping_status = get_option("default_ping_status");
+						break;
+				}
 			}
 		}
 
@@ -1183,9 +1303,9 @@ class wp_xmlrpc_server extends IXR_Server {
 
 	  if ( ('publish' == $post_status) ) {
 	  	if ( ( 'page' == $post_type ) && !current_user_can('publish_pages') )
-	  		return new IXR_Error(401, 'Sorry, you do not have the right to publish this page.');
+	  		return new IXR_Error(401, __('Sorry, you do not have the right to publish this page.'));
 	  	else if ( !current_user_can('publish_posts') )
-		  	return new IXR_Error(401, 'Sorry, you do not have the right to publish this post.');
+		  	return new IXR_Error(401, __('Sorry, you do not have the right to publish this post.'));
 	  }
 
 	  if ($post_more) {
@@ -1196,16 +1316,12 @@ class wp_xmlrpc_server extends IXR_Server {
 	  if ( is_array($to_ping) )
 	  	$to_ping = implode(' ', $to_ping);
 
-      if(isset($content_struct["mt_allow_comments"])) {
-		$comment_status = $content_struct["mt_allow_comments"];
-      }
-	  
 	  // Do some timestamp voodoo
 	  $dateCreatedd = $content_struct['dateCreated'];
 	  if (!empty($dateCreatedd)) {
 	    $dateCreated = $dateCreatedd->getIso();
 	    $post_date     = get_date_from_gmt(iso8601_to_datetime($dateCreated));
-	    $post_date_gmt = iso8601_to_datetime($dateCreated, GMT);
+	    $post_date_gmt = iso8601_to_datetime($dateCreated . "Z", GMT);
 	  } else {
 	    $post_date     = $postdata['post_date'];
 	    $post_date_gmt = $postdata['post_date_gmt'];
@@ -1216,7 +1332,7 @@ class wp_xmlrpc_server extends IXR_Server {
 
 	  $result = wp_update_post($newpost);
 	  if (!$result) {
-	    return new IXR_Error(500, 'Sorry, your entry could not be edited. Something wrong happened.');
+	    return new IXR_Error(500, __('Sorry, your entry could not be edited. Something wrong happened.'));
 	  }
 	  $this->attach_uploads( $ID, $post_content );
 
@@ -1245,7 +1361,7 @@ class wp_xmlrpc_server extends IXR_Server {
 
 	  if ($postdata['post_date'] != '') {
 
-	    $post_date = mysql2date('Ymd\TH:i:s', $postdata['post_date']);
+	    $post_date = mysql2date('Ymd\TH:i:s\Z', $postdata['post_date_gmt']);
 
 	    $categories = array();
 	    $catids = wp_get_post_categories($post_ID);
@@ -1285,7 +1401,7 @@ class wp_xmlrpc_server extends IXR_Server {
 
 	    return $resp;
 	  } else {
-	  	return new IXR_Error(404, 'Sorry, no such post.');
+	  	return new IXR_Error(404, __('Sorry, no such post.'));
 	  }
 	}
 
@@ -1307,13 +1423,13 @@ class wp_xmlrpc_server extends IXR_Server {
 		$posts_list = wp_get_recent_posts($num_posts);
 
 		if (!$posts_list) {
-			$this->error = new IXR_Error(500, 'Either there are no posts, or something went wrong.');
+			$this->error = new IXR_Error(500, __('Either there are no posts, or something went wrong.'));
 			return $this->error;
 		}
 
 		foreach ($posts_list as $entry) {
 
-			$post_date = mysql2date('Ymd\TH:i:s', $entry['post_date']);
+			$post_date = mysql2date('Ymd\TH:i:s\Z', $entry['post_date_gmt']);
 			$categories = array();
 			$catids = wp_get_post_categories($entry['ID']);
 			foreach($catids as $catid) {
@@ -1412,6 +1528,21 @@ class wp_xmlrpc_server extends IXR_Server {
 		$type = $data['type'];
 		$bits = $data['bits'];
 
+		logIO('O', '(MW) Received '.strlen($bits).' bytes');
+
+		if ( !$this->login_pass_ok($user_login, $user_pass) )
+			return $this->error;
+
+		set_current_user(0, $user_login);
+		if ( !current_user_can('upload_files') ) {
+			logIO('O', '(MW) User does not have upload_files capability');
+			$this->error = new IXR_Error(401, __('You are not allowed to upload files to this site.'));
+			return $this->error;
+		}
+
+		if ( $upload_err = apply_filters( "pre_upload_error", false ) )
+			return new IXR_Error(500, $upload_err);
+
 		if(!empty($data["overwrite"]) && ($data["overwrite"] == true)) {
 			// Get postmeta info on the object.
 			$old_file = $wpdb->get_row("
@@ -1430,21 +1561,6 @@ class wp_xmlrpc_server extends IXR_Server {
 			$name = "wpid{$old_file->ID}-{$filename}";
 		}
 
-		logIO('O', '(MW) Received '.strlen($bits).' bytes');
-
-		if ( !$this->login_pass_ok($user_login, $user_pass) )
-			return $this->error;
-
-		set_current_user(0, $user_login);
-		if ( !current_user_can('upload_files') ) {
-			logIO('O', '(MW) User does not have upload_files capability');
-			$this->error = new IXR_Error(401, 'You are not allowed to upload files to this site.');
-			return $this->error;
-		}
-
-		if ( $upload_err = apply_filters( "pre_upload_error", false ) )
-			return new IXR_Error(500, $upload_err);
-
 		$upload = wp_upload_bits($name, $type, $bits, $overwrite);
 		if ( ! empty($upload['error']) ) {
 			$errorString = 'Could not write file ' . $name . ' (' . $upload['error'] . ')';
@@ -1492,13 +1608,13 @@ class wp_xmlrpc_server extends IXR_Server {
 		$posts_list = wp_get_recent_posts($num_posts);
 
 		if (!$posts_list) {
-			$this->error = new IXR_Error(500, 'Either there are no posts, or something went wrong.');
+			$this->error = new IXR_Error(500, __('Either there are no posts, or something went wrong.'));
 			return $this->error;
 		}
 
 		foreach ($posts_list as $entry) {
 
-			$post_date = mysql2date('Ymd\TH:i:s', $entry['post_date']);
+			$post_date = mysql2date('Ymd\TH:i:s\Z', $entry['post_date_gmt']);
 
 			$struct[] = array(
 				'dateCreated' => new IXR_Date($post_date),
@@ -1595,7 +1711,7 @@ class wp_xmlrpc_server extends IXR_Server {
 
 		set_current_user(0, $user_login);
 		if ( !current_user_can('edit_post', $post_ID) )
-			return new IXR_Error(401, 'Sorry, you can not edit this post.');
+			return new IXR_Error(401, __('Sorry, you can not edit this post.'));
 
 		foreach($categories as $cat) {
 			$catids[] = $cat['categoryId'];
@@ -1636,7 +1752,7 @@ class wp_xmlrpc_server extends IXR_Server {
 		$actual_post = wp_get_single_post($post_ID, ARRAY_A);
 
 		if (!$actual_post) {
-			return new IXR_Error(404, 'Sorry, no such post.');
+			return new IXR_Error(404, __('Sorry, no such post.'));
 		}
 
 		$comments = $wpdb->get_results("SELECT comment_author_url, comment_content, comment_author_IP, comment_type FROM $wpdb->comments WHERE comment_post_ID = $post_ID");
@@ -1677,7 +1793,7 @@ class wp_xmlrpc_server extends IXR_Server {
 
 		set_current_user(0, $user_login);
 		if ( !current_user_can('edit_post', $post_ID) )
-			return new IXR_Error(401, 'Sorry, you can not edit this post.');
+			return new IXR_Error(401, __('Sorry, you can not edit this post.'));
 
 		$postdata = wp_get_single_post($post_ID,ARRAY_A);
 
@@ -1718,7 +1834,7 @@ class wp_xmlrpc_server extends IXR_Server {
 		// Check if the page linked to is in our site
 		$pos1 = strpos($pagelinkedto, str_replace(array('http://www.','http://','https://www.','https://'), '', get_option('home')));
 		if( !$pos1 )
-				return new IXR_Error(0, 'Is there no link to us?');
+				return new IXR_Error(0, __('Is there no link to us?'));
 
 		// let's find which post is linked to
 		// FIXME: does url_to_postid() cover all these cases already?
@@ -1771,7 +1887,7 @@ class wp_xmlrpc_server extends IXR_Server {
 	  		return new IXR_Error(33, 'The specified target URL cannot be used as a target. It either doesn\'t exist, or it is not a pingback-enabled resource.');
 
 		if ( $post_ID == url_to_postid($pagelinkedfrom) )
-			return new IXR_Error(0, 'The source URL and the target URL cannot both point to the same resource.');
+			return new IXR_Error(0, __('The source URL and the target URL cannot both point to the same resource.'));
 
 		// Check if pings are on
 		if ( 'closed' == $post->ping_status )
@@ -1781,7 +1897,7 @@ class wp_xmlrpc_server extends IXR_Server {
 		$result = $wpdb->get_results("SELECT * FROM $wpdb->comments WHERE comment_post_ID = '$post_ID' AND comment_author_url = '$pagelinkedfrom'");
 
 		if ( $wpdb->num_rows ) // We already have a Pingback from this URL
-	  		return new IXR_Error(48, 'The pingback has already been registered.');
+	  		return new IXR_Error(48, __('The pingback has already been registered.'));
 
 		// very stupid, but gives time to the 'from' server to publish !
 		sleep(1);
@@ -1789,7 +1905,7 @@ class wp_xmlrpc_server extends IXR_Server {
 		// Let's check the remote site
 		$linea = wp_remote_fopen( $pagelinkedfrom );
 		if ( !$linea )
-	  		return new IXR_Error(16, 'The source URL does not exist.');
+	  		return new IXR_Error(16, __('The source URL does not exist.'));
 
 		// Work around bug in strip_tags():
 		$linea = str_replace('<!DOC', '<DOC', $linea);
@@ -1799,7 +1915,7 @@ class wp_xmlrpc_server extends IXR_Server {
 		preg_match('|<title>([^<]*?)</title>|is', $linea, $matchtitle);
 		$title = $matchtitle[1];
 		if ( empty( $title ) )
-			return new IXR_Error(32, 'We cannot find a title on that page.');
+			return new IXR_Error(32, __('We cannot find a title on that page.'));
 
 		$linea = strip_tags( $linea, '<a>' ); // just keep the tag we need
 
@@ -1835,7 +1951,7 @@ class wp_xmlrpc_server extends IXR_Server {
 		}
 
 		if ( empty($context) ) // Link to target not found
-			return new IXR_Error(17, 'The source URL does not contain a link to the target URL, and so cannot be used as a source.');
+			return new IXR_Error(17, __('The source URL does not contain a link to the target URL, and so cannot be used as a source.'));
 
 		$pagelinkedfrom = preg_replace('#&([^amp\;])#is', '&amp;$1', $pagelinkedfrom);
 
@@ -1882,7 +1998,7 @@ class wp_xmlrpc_server extends IXR_Server {
 
 		if (!$actual_post) {
 			// No such post = resource not found
-	  		return new IXR_Error(32, 'The specified target URL does not exist.');
+	  		return new IXR_Error(32, __('The specified target URL does not exist.'));
 		}
 
 		$comments = $wpdb->get_results("SELECT comment_author_url, comment_content, comment_author_IP, comment_type FROM $wpdb->comments WHERE comment_post_ID = $post_ID");
